
package examples.jmx;

/**
 * Sequoia: Database clustering technology.
 * Copyright (C) 2002-2004 French National Institute For Research In Computer
 * Science And Control (INRIA).
 * Copyright (C) 2005 AmicoSoft, Inc. dba Emic Networks
 * Contact: sequoia@continuent.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 *
 * Initial developer(s): Marc Wick.
 * Contributor(s): 
 */
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.management.MBeanServerConnection;
import javax.management.Notification;
import javax.management.NotificationListener;
import javax.management.ObjectName;
import javax.management.remote.JMXConnectionNotification;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;
import javax.naming.Context;

import org.continuent.sequoia.common.jmx.JmxConstants;

/**
 * This class defines a jmx notification listener. It will register with the
 * sequoia controller and recieve jmx notifications about all sequioa events.
 * 
 * @author <a href="mailto:marc.wick@monte-bre.ch">Marc Wick </a>
 * @version 1.0
 */
public class SequoiaNotificationListener implements NotificationListener
{

  private String                port          = "8091";

  private String                host          = "localhost";

  private String                virtualDB     = "rubis";

  private String                adminUserName = "admin";

  private String                adminPassword = "sequioa";

  private JMXConnector          connector;

  private MBeanServerConnection server;

  private Logger                logger        = Logger.global;

  private void setLogger(Logger pLogger)
  {
    if (pLogger == null)
    {
      throw new Error("Logger may not be null");
    }
    logger = pLogger;
  }

  /**
   * {@inheritDoc}
   * 
   * @see javax.management.NotificationListener#handleNotification(javax.management.Notification,
   *      java.lang.Object)
   */
  public void handleNotification(Notification notification, Object handback)
  {
    logger.info("\nReceived sequoia notification: " + notification);

    if (notification instanceof JMXConnectionNotification)
    {
      JMXConnectionNotification connectionNotification = (JMXConnectionNotification) notification;
      logger.info("sequoia jmx connection id "
          + connectionNotification.getConnectionId() + ", type="
          + notification.getType());
    }

    if (JMXConnectionNotification.FAILED.equals(notification.getType())
        || JMXConnectionNotification.CLOSED.equals(notification.getType()))
    {
      try
      {
        logger.info("sequoia about to reconnect because of "
            + notification.getType());
        reconnect();
        logger.info("sequoia jmx reconnect ok");
      }
      catch (Exception e)
      {
        logger.log(Level.INFO, "jmx reconnect failed", e);
      }
    }
  }

  /**
   * reconnect to the controller in case the controller was shut down. We try to
   * reconnect for some time.
   * 
   * @throws Exception if an error occurs
   */
  public void reconnect() throws Exception
  {
    long waitTime = 15000;
    for (int retries = 0; retries < 10; retries++)
    {
      try
      {
        connect();
        logger.info("jmx sequoia reconnect successful");
        return;
      }
      catch (Exception e)
      {
        logger.log(Level.INFO, "jmx sequoia reconnect failed", e);
        System.out.println("reconnect failed. (" + retries + ")");
      }
      System.out.println("waiting for next try " + waitTime / 1000 + " sec ");
      Thread.sleep(waitTime);

      // duplicate wait time with every iteration
      waitTime = waitTime * 2;
    }
    throw new Exception(
        "we stop trying to reconnect maximal number of attempts exceeded");
  }

  /**
   * connect to the sequoia controller and register this class as listener
   * 
   * @throws Exception if an error occurs
   */
  public void connect() throws Exception
  {
    JMXServiceURL address = new JMXServiceURL("rmi", host, 0, "/jndi/jrmp");

    Map environment = new HashMap();
    environment.put(Context.INITIAL_CONTEXT_FACTORY,
        "com.sun.jndi.rmi.registry.RegistryContextFactory");
    environment.put(Context.PROVIDER_URL, "rmi://" + host + ":" + port);

    // use username and password for authentication of connections
    // with the controller, the values are compared to the ones
    // specified in the controller.xml config file.
    // this line is not required if no username/password has been configered
    // environment.put(JMXConnector.CREDENTIALS, PasswordAuthenticator
    // .createCredentials("jmxuser", "jmxpassword"));

    connector = JMXConnectorFactory.connect(address, environment);

    logger.info("new jmx connection established to sequoia controller on port "
        + port);

    // we want to be informed about connection problems
    connector.addConnectionNotificationListener(this, null, null);
    server = connector.getMBeanServerConnection();

    // ObjectName db = JmxConstants.getVirtualDbObjectName(virtualDB);
    ObjectName db = JmxConstants.getVirtualDataBaseObjectName(virtualDB);

    // we are registering for notifications emitted by the controller
    // for example if a backend is disabled
    server.addNotificationListener(db, this, null, null);
    logger.info("addNotificationListener for " + db.toString());
  }

  /**
   * main class as example. In a real application this part will probably be
   * included in the application itself or in some other monitoring program.
   * 
   * @param args command line arguments (ignored)
   * @throws Exception if an error occurs
   */
  public static void main(String[] args) throws Exception
  {
    SequoiaNotificationListener monitor = new SequoiaNotificationListener();
    monitor.connect();
    // monitor for five minutes as a test.
    Thread.sleep(5 * 60000);
  }

  /**
   * @return Returns the adminPassword.
   */
  public String getAdminPassword()
  {
    return adminPassword;
  }

  /**
   * @param adminPassword The adminPassword to set.
   */
  public void setAdminPassword(String adminPassword)
  {
    this.adminPassword = adminPassword;
  }

  /**
   * @return Returns the adminUserName.
   */
  public String getAdminUserName()
  {
    return adminUserName;
  }

  /**
   * @param adminUserName The adminUserName to set.
   */
  public void setAdminUserName(String adminUserName)
  {
    this.adminUserName = adminUserName;
  }

  /**
   * @return Returns the host.
   */
  public String getHost()
  {
    return host;
  }

  /**
   * @param host The host to set.
   */
  public void setHost(String host)
  {
    this.host = host;
  }

  /**
   * @return Returns the port.
   */
  public String getPort()
  {
    return port;
  }

  /**
   * @param port The port to set.
   */
  public void setPort(String port)
  {
    this.port = port;
  }
}
